import datetime
import hashlib
import multiprocessing
import time
import json
import redis
import crawler_baidu
import crawler_boss
import random
import requests
import cyaml


WAIT_SPIDER_BAIKE_COMPANY_LIST = "sw:wait:spider:bk:company:list"
WAIT_SPIDER_BOSS_COMPANY_LIST = "sw:wait:spider:boss:company:list"


# 获取需要爬数据的企业 百度1
def GetCompany(types):
    response = requests.get(
        cyaml.data[cyaml.data["env"]]["url"] + cyaml.data["php-api"]["getcompany"] + "?type=" + str(types),
        headers={"Content-Type": "application/json"})
    if response.status_code == 200:
        response_data = response.json()
        # if response_data.get('name') is not None:
        # return "上海临方股权投资管理有限公司","https://www.zhipin.com/web/geek/job?query=%E5%A4%A7%E6%97%8F%E6%BF%80%E5%85%89&city=100010000"
        # return "上海临方股权投资管理有限公司","https://baike.baidu.com/item/%E4%B8%8A%E6%B5%B7%E4%B8%B4%E6%96%B9%E8%82%A1%E6%9D%83%E6%8A%95%E8%B5%84%E7%AE%A1%E7%90%86%E6%9C%89%E9%99%90%E5%85%AC%E5%8F%B8?fromModule=lemma_search-box"
        return response_data["data"]["company_name"], response_data["data"]["url"]
    return "", ""


# 重新放入采集队列
def DoSpiderCompany(company, crawler_website):
    if crawler_website == "baike":
        redis_key = WAIT_SPIDER_BAIKE_COMPANY_LIST
    elif crawler_website == "boss":
        redis_key = WAIT_SPIDER_BOSS_COMPANY_LIST
    else:
        redis_key = ""
    if redis_key != "":
        r = GetRedisConnect()
        # 先删除，再添加
        r.lrem(redis_key, -1, company)
        r.rpush(redis_key, company)
    return 200


# 回传爬虫获取到的数据
def SaveCompanyData(name, types, intro):
    my_obj = {"intro": intro}
    post_data = {
        "name": name,
        "content": json.dumps(my_obj),
        "type": types
    }
    json_data = json.dumps(post_data)
    response = requests.post(cyaml.data[cyaml.data["env"]]["url"] + cyaml.data["php-api"]["getcompany"], data=json_data,
                             headers={"Content-Type": "application/json"})
    # Log("保存数据时候的返回：" + response)
    # 检查响应状态码
    if response.status_code == 200:
        response_data = response.json()
        if response_data["code"] == 200:
            return name
        else:
            return ""
    else:
        return ""


# 启动爬虫
def CrawlerLaunch():
    while True:
        now = datetime.datetime.now()
        randomtime = random.randint(3, cyaml.data[cyaml.data["env"]]["randomtime"])
        print(str(now) + ":启动等待时间" + str(randomtime))
        time.sleep(randomtime)
        crawler_baidu.CrawlerBaidu()
        crawler_boss.CrawlerBoss()


def Log(dataS):
    now = datetime.datetime.now()
    date_time = now.strftime('%Y-%m-%d %H:%M:%S')
    with open("log/" + str(datetime.date.today()) + ".txt", "a+") as f:
        f.write(date_time + "：" + dataS + "\n")
        f.close()


def GetRedisConnect():
    conf = cyaml.data[cyaml.data["env"]]
    r = redis.StrictRedis(host=conf['redis_host'], port=conf['redis_port'], db=conf['redis_db'],
                          password=conf['redis_password'], client_name=conf['redis_client'])
    return r


def Md5Encode(value: str):
    # 创建md5对象
    md5 = hashlib.md5()
    # 更新md5对象
    md5.update(value.encode("utf-8"))
    # 获取md5值
    result = md5.hexdigest()
    return result


def CheckUpperLimit(name, crawler_org):
    # 未获取到百科资料，并且未达到最大次数
    r = GetRedisConnect()
    redis_key = name + crawler_org
    # 对公司名称进行md5转换为唯一id
    identifier = Md5Encode(redis_key)
    # 查询当前已查询次数
    r_data = r.get(identifier)
    if r_data is not None:
        upper_limit = int(cyaml.data[cyaml.data["env"]]['upper_limit'])
        json_data = json.loads(r_data)
        num = int(json_data.get(crawler_org, 0))
        Log(name + ":" + crawler_org + "数据" + ":当前次数为：" + str(num))
        # 查询次数达到设定值，通知dwp已爬取无内容
        if num >= upper_limit:
            if crawler_org == "baike":
                SaveCompanyData(name, 1, "")
            elif crawler_org == "boss":
                SaveCompanyData(name, 2, "")
            # 删除达到设定值的key
            r.delete(identifier)
            Log(name + ":" + crawler_org + "数据" + ":已达到上限，同步dwp修改状态为以爬取无内容，当前次数为：" + str(num))
        else:
            num += 1
            json_data[crawler_org] = num
            r.set(identifier, json.dumps(json_data))
            Log(name + "未达到上限，次数+1，当前次数为：" + str(num))
    else:
        # 当前redis没有该key值，初始化
        json_dict = {crawler_org: "1"}
        r.set(identifier, json.dumps(json_dict))
        num = 1
        Log(name + ":" + crawler_org + "数据，数据查询无内容，当前查询次数：" + str(num))
    return num


if __name__ == "__main__":
    p = multiprocessing.Process(target=CrawlerLaunch)
    p.start()
    p.join()
